@@ -10,7 +10,7 @@ import (
 )
 
 type Cache interface {
-	Init(home string)
+	Init(filePath string, persist bool)
 	Close()
 	// Gets the value for a given key.
 	// Returns the value and a boolean indicating if the key was found.
@@ -27,9 +27,9 @@ const (
 	FileName = "omp.cache"
 )
 
-var SessionFileName = fmt.Sprintf("%s.%s", FileName, pid())
+var SessionFileName = fmt.Sprintf("%s.%s", FileName, sessionID())
 
-func pid() string {
+func sessionID() string {
 	pid := os.Getenv("POSH_PID")
 	if len(pid) == 0 {
 		log.Debug("POSH_PID not set, using process pid")
